<?php

require_once('db.php');

function passwd($login,$pass)
{
  return md5(md5($pass).$login);
}

function format_ajax($data)
{
  header('Content-type: application/json; charset=UTF-8');
  echo json_encode($data);
  exit;
}

function format_xml($errmsg)
{
  header("Cache-Control: no-cache");
  header("Pragma: no-cache");
  header("Expires: ".gmdate("D, d M Y H:i:s",time()+(-1*60))." GMT");
  header("Content-type: text/xml");

  echo "<?xml version='1.0' encoding='UTF-8'?".">\n";
  echo "\n<ajax-response><response type='object'>";
  echo "\n<error>";
  echo "\n".htmlspecialchars($errmsg);
  echo "\n</error>";
  echo "\n</response></ajax-response>";
  exit;
}

function check_permissions($name)
{
  global $db;

  $parts = explode('__', $name);
  $form = $parts[0] . '|' . $parts[1];

  $sql = sprintf("SELECT count(*) AS count FROM sys_mask "
		 . "JOIN sys_menu ON sys_mask.menu = sys_menu.id "
		 . "JOIN sys_group_mask ON sys_mask.id = sys_group_mask.mask "
		 . "WHERE sys_group_mask.gid = %d AND fname = %s",
		 $_SESSION['sys']['group'], $db->quote($form));

  $sth = $db->query($sql);

  if ($sth === false) {
    error_log('Unauthorised access to ' . $form);
    return false;
  }

  $row = $sth->fetch();
  if ($row === false) {
    error_log('Unauthorised access to ' . $form);
    return false;
  }

  if ($row['count'] == 0) {
    error_log('Unauthorised access to ' . $form);
    return false;
  }

  return true;
}

function check_session()
{
  /* table data */
  if (substr($_SERVER["SCRIPT_FILENAME"],-17) == '/ricoXMLquery.php' &&
      !empty($_GET['id']) && substr($_GET['id'],0,5) == 'grid_') {
    if (check_permissions(substr($_GET['id'],5)))
      return true;
    else {
      if (empty($_SESSION['sys']['login']))
	format_xml("No permission to access data.\nNo active session found.\nYou may need to re-login.");
      else
	format_xml('No permission to access data');
    }
  }

  /* table connections */
  if (substr($_SERVER["SCRIPT_FILENAME"],-25) == '/ricoUpdateConnection.php' &&
      !empty($_GET['id']) && substr($_GET['id'],0,5) == 'grid_') {
    if (check_permissions(substr($_GET['id'],5)))
      return true;
    else
      format_ajax(array('error' => 'No permission to access data'));
  }

  /* ajax calls */
  if (substr($_SERVER["SCRIPT_FILENAME"],-9) == '/ajax.php' &&
      !empty($_POST['source'])) {
    if (check_permissions($_POST['source']))
      return true;
    else {
      if (empty($_SESSION['sys']['login']))
	format_ajax(array('error' => "No permission to access data.\nNo active session found.\nYou need to re-login.",
			  'logout' => true));
      else
	format_ajax(array('error' => 'No permission to access data'));
    }
  }

  if (!empty($_SESSION['sys']['login']) && !empty($_GET['logout'])) {
    session_destroy();
    header('Location: ./?login=true');
    exit();
  }

  if (substr($_SERVER["SCRIPT_FILENAME"],-10) == '/index.php' &&
      !empty($_POST['login']) && !empty($_POST['passwd'])) {
    require_once('lib/login.php');
    if (check_passwd()) {
      header('Location: ./');
      exit();
    }
  }

  if (empty($_SESSION['sys']['login']) && empty($_GET['login'])) {
    header('Location: ./?login=true');
    exit();
  }

  /* regular mask */
  if (!empty($_GET['mask'])) {
    if (check_permissions($_GET['mask']))
      return true;
    else {
      header('Location: ./');
      exit();
    }
  }

  return true;
}

function sanitise_filename($file)
{
  return str_replace('./','x',$file);
}

function load_mask($name)
{
  global $mask;
  global $jscode;
  global $style;

  $name = sanitise_filename($name);
  list($module,$fname) = explode('__', $name);
  $_SESSION['module'] = $module;

  $file = $_SESSION['sys']['basedir'] . 'masks/' . $module . '/' . $fname . '.php';

  if (!file_exists($file))
    return false;

  include_once($file);

  return true;
}

function load_js($jsfiles, $jscode)
{
  $ret = '';
  foreach ($jsfiles as $file)
    $ret .= sprintf('<script type="text/javascript" src="%s"></script>'."\n", $file);

  if (!empty($jscode))
    $ret .= sprintf('<script type="text/javascript">'."\n%s\n</script>\n", implode("\n",$jscode));

  return $ret;
}

function load_style($style)
{
  $ret = '';
  if (!empty($style))
    $ret .= sprintf('<style type="text/css">'."\n%s\n</style>\n", implode("\n",$style));

  return $ret;
}

function process()
{
  if (!empty($_GET['login'])) {
    require_once('lib/login.php');
    return mask_login();
  }

  if (!empty($_GET['mask'])) {
    require_once('lib/mask.php');
    return mask($_GET['mask']);
  }

  $ret = '';
  $fname = $_SESSION['sys']['basedir'] . 'data/misc/start.html';
  if (file_exists($fname))
    return file_get_contents($fname);
  
  return $ret;
}

function debug_log($text)
{
  global $debug_info;

  $debug_info .= '<br>' . $text;
}

function debug_info()
{
  global $jsfiles;
  global $debug_info;

  if (DEBUG !== true) return '';

  $jsfiles[] = 'lib/debug_joey.js';

  $html = '<div style="background: #DDD; margin: 5px; padding-left: 4px; border: 1px solid #AAA;clear:both;">';
  $html .= "\n<pre>\n\$_SESSION = " . var_export($_SESSION,true) . "\n";
  $html .= "\n\$_COOKIE = " . var_export($_COOKIE,true) . "\n</pre>\n";
  $html .= $debug_info;
  $html .= '</div>';
  return $html;
}

function grid_sql_join($mask)
{
  $ret = '';
  if (array_key_exists('join', $mask)) {
    foreach ($mask['join'] as $line) {
      if (strtolower(substr($line,0,4)) == 'left')
	$ret .= ' LEFT JOIN ' . substr($line,5);
      else
	$ret .= ' JOIN ' . $line;
    }
  }
  return $ret;
}

function grid_lookup_sql($table,$id,$text)
{
  global $db;

  if (empty($_GET['mask'])) return;

  $sql = "SELECT $id,$text FROM $table ORDER BY $text";
  $sth = $db->query($sql);

  $result = array();
  while ($row = $sth->fetch())
    $result[] = sprintf("%d: '%s'", $row[$id], $row[$text]);

  return '{' . implode(', ', $result) . '}';
}

function grid_sql($name, $mask)
{
  $i = 0;
  $fields = array();
  foreach ($mask['list'] as $field => $data) {
    if (array_key_exists('sql', $data))
      $fields[] = $data['sql'] . ' AS ' . $field;
    else
      $fields[] = $field;
    if (array_key_exists('distinct', $data))
      $_SESSION['grid_' . $name.'_distinct_'.$i] = $data['distinct'];
    $i++;
  }

  $_SESSION['grid_' . $name] = sprintf("SELECT %s FROM %s",
				       implode(',', $fields), $mask['table']);
  if (array_key_exists('join', $mask)) $_SESSION['grid_' . $name] .= grid_sql_join($mask);
  if (array_key_exists('where', $mask)) $_SESSION['grid_' . $name] .= ' WHERE ' . $mask['where'];
}

?>
